# Word Search Animation from 09/27/18 Lecture

from tkinter import *

# Word Search Solver Algorithm

def wordSearch(board, word):
    for row in range(len(board)):
        for col in range(len(board[0])):
            result = wordSearchFromPosition(board, word, row, col)
            if result != None:
                return result
    return None

def wordSearchFromPosition(board, word, row, col):
    for drow in [-1, 0, 1]:
        for dcol in [-1, 0, 1]:
            if drow == dcol == 0:
                continue
            result = wordSearchFromPositionInDir(board, word, 
                                                 row, col, drow, dcol)
            if result != None:
                return result
    return None

def wordSearchFromPositionInDir(board, word, row, col, drow, dcol):
    curRow, curCol = row, col
    for i in range(len(word)):
        curRow = row + i*drow
        curCol = col + i*dcol
        if curRow < 0 or curRow >= len(board) or \
           curCol < 0 or curCol >= len(board[row]):
            return None
        if board[curRow][curCol] != word[i]:
            return None
    return [(row, col), (curRow, curCol)]

# Word Search Animation

def init(data):
    data.board = [   [ 'd', 'o', 'g' ],
                        [ 't', 'a', 'c' ],
                        [ 'o', 'a', 't' ],
                        [ 'u', 'r', 'k' ],
                    ]
    data.solutions = [ "dog", "cat", "tad"]
    cellWidth = data.width//len(data.board[0])
    cellHeight = data.height//len(data.board)
    data.cellSize = min(cellWidth, cellHeight)
    data.selected = []
    data.solvedWords = []

def foundWord(data):
    word = ""
    for point in data.selected:
        (row, col) = point
        word += data.board[row][col]
    return word in data.solutions

def mousePressed(event, data):
    # Note: this only works if the user clicks the cells in the correct order.
    # Fixing that is a task for another day...
    row = event.y // data.cellSize
    col = event.x // data.cellSize
    if 0 <= row < len(data.board) and \
       0 <= col < len(data.board[0]):
        data.selected.append((row, col))
        if foundWord(data):
            data.solvedWords.append([data.selected[0], data.selected[-1]])
            data.selected = []
    else:
        data.selected = []

def keyPressed(event, data):
    if event.char == "s":
        for word in data.solutions:
            data.solvedWords.append(wordSearch(data.board, word))

def drawBoard(canvas, data):
    for row in range(len(data.board)):
        for col in range(len(data.board[row])):
            if (row, col) in data.selected:
                color = "yellow"
            else:
                color = "turquoise"
            left = col*data.cellSize
            top = row*data.cellSize
            canvas.create_rectangle(left, top, 
                                    left + data.cellSize, top + data.cellSize,
                                    fill=color)
            canvas.create_text(left + data.cellSize/2, top + data.cellSize/2,
                            text=data.board[row][col],
                            font="Arial " + str(int(data.cellSize/2)) + " bold")

def drawLines(canvas, data):
    for line in data.solvedWords:
        left = line[0][1] * data.cellSize + data.cellSize/2
        top = line[0][0] * data.cellSize + data.cellSize/2
        right = line[1][1] * data.cellSize + data.cellSize/2
        bottom = line[1][0] * data.cellSize + data.cellSize/2
        canvas.create_line(left, top, right, bottom, width=5, fill="white")
    
def drawSolutions(canvas, data):
    margin = 20
    left = data.cellSize * len(data.board[0]) + margin
    for i in range(len(data.solutions)):
        canvas.create_text(left, data.cellSize*i,
                           text=data.solutions[i], anchor="nw",
                           font="Arial " + str(int(data.cellSize/4)) + " bold")

def redrawAll(canvas, data):
    drawBoard(canvas, data)
    drawLines(canvas, data)
    drawSolutions(canvas, data)

####################################
# use the run function as-is
####################################

def run(width=300, height=300):
    def redrawAllWrapper(canvas, data):
        canvas.delete(ALL)
        canvas.create_rectangle(0, 0, data.width, data.height,
                                fill='white', width=0)
        redrawAll(canvas, data)
        canvas.update()    

    def mousePressedWrapper(event, canvas, data):
        mousePressed(event, data)
        redrawAllWrapper(canvas, data)

    def keyPressedWrapper(event, canvas, data):
        keyPressed(event, data)
        redrawAllWrapper(canvas, data)

    # Set up data and call init
    class Struct(object): pass
    data = Struct()
    data.width = width
    data.height = height
    root = Tk()
    root.resizable(width=False, height=False) # prevents resizing window
    init(data)
    # create the root and the canvas
    canvas = Canvas(root, width=data.width, height=data.height)
    canvas.configure(bd=0, highlightthickness=0)
    canvas.pack()
    # set up events
    root.bind("<Button-1>", lambda event:
                            mousePressedWrapper(event, canvas, data))
    root.bind("<Key>", lambda event:
                            keyPressedWrapper(event, canvas, data))
    redrawAll(canvas, data)
    # and launch the app
    root.mainloop()  # blocks until window is closed
    print("bye!")

run(600, 600)